@@ -17,6 +17,8 @@
 #ifndef __LIBSTROPHE_OSTYPES_H__
 #define __LIBSTROPHE_OSTYPES_H__
 
+#include <stddef.h>     /* size_t */
+
 #if defined (_MSC_VER) && _MSC_VER < 1600
 typedef signed char int8_t;
 typedef short int int16_t;
@@ -27,6 +29,17 @@ typedef unsigned char uint8_t;
 typedef unsigned short int uint16_t;
 typedef unsigned int uint32_t;
 typedef unsigned __int64 uint64_t; 
+
+#ifndef UINT16_MAX
+#define UINT16_MAX ((uint16_t)0xffff)
+#endif /* UINT16_MAX */
+#ifndef UINT32_MAX
+#define UINT32_MAX ((uint32_t)0xffffffff)
+#endif /* UINT32_MAX */
+#ifndef SIZE_MAX
+#define SIZE_MAX UINT32_MAX
+#endif /* SIZE_MAX */
+
 #else
 #include <stdint.h>
 #endif
